<?php

namespace Drupal\views_expandable_table\Plugin\views\style;

use Drupal\Core\Form\FormStateInterface;
use Drupal\views\Plugin\views\style\Table;

/**
 * Style plugin to render each item as a row in an expandable table.
 *
 * @ingroup views_style_plugins
 *
 * @ViewsStyle(
 *   id = "views_view_expandabletable",
 *   title = @Translation("Expandable Table"),
 *   help = @Translation("Displays rows in an expandable table."),
 *   theme = "views_view_expandabletable",
 *   display_types = {"normal"}
 * )
 */
class ExpandableTable extends Table {

  /**
   * {@inheritdoc}
   */
  protected function defineOptions() {
    $options = parent::defineOptions();
    $options['triggerable_row'] = ['default' => TRUE];
    $options['toggle_location'] = ['default' => 'last'];
    $options['element_location'] = ['default' => 'after'];

    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    parent::buildOptionsForm($form, $form_state);

    $form['triggerable_row'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Use triggerable row'),
      '#default_value' => $this->options['triggerable_row'],
      '#description' => $this->t('If this is checked, the entire row will be triggered. If not, a special element will be added to the end of the specified forst or last visible @td element.', ['@td' => '<td>']),
    ];

    $form['toggle_location'] = [
      '#type' => 'select',
      '#title' => $this->t('Location of the toggle element'),
      '#default_value' => $this->options['toggle_location'],
      '#options' => [
        'first' => $this->t('First visible'),
        'last' => $this->t('Last visible'),
      ],
      '#description' => $this->t('<p>What column should the triggerable element appear in?</p> <p>NOTE: If this view uses views bulk operations, the next available column will be used instead.</p>'),
      '#states' => [
        'visible' => [
          'input[name="style_options[triggerable_row]"]' => ['checked' => FALSE],
        ],
      ],
    ];

    $form['element_location'] = [
      '#type' => 'select',
      '#title' => $this->t('Position of the toggle element'),
      '#default_value' => $this->options['element_location'],
      '#options' => [
        'before' => $this->t('Before content'),
        'after' => $this->t('After content'),
      ],
      '#description' => $this->t('What position should the element appear in?'),
      '#states' => [
        'visible' => [
          'input[name="style_options[triggerable_row]"]' => ['checked' => FALSE],
        ],
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function render() {
    $build = parent::render();
    $build['#attached']['library'][] = 'views_expandable_table/expandable_table';
    $build['#attached']['drupalSettings']['views_expandable_table']['triggerable_row'] = $this->options['triggerable_row'];
    return $build;
  }

}
